<?xml version='1.0' encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl='http://www.w3.org/1999/XSL/Transform' version='1.0'>  
<xsl:output method="html" encoding="ISO-8859-1" />

<xsl:template match="doc">
<html>
 <xsl:apply-templates />
</html>
</xsl:template>

<!-- Header ========================================================= -->

<xsl:template match="head">
 <head>
  <xsl:apply-templates />
  <!-- <xsl:call-template name="css" />-->
 </head>
</xsl:template>

<xsl:template match="title">
 <title><xsl:apply-templates /></title>
</xsl:template>

<xsl:template match="author">
 <meta name="AUTHOR">
  <xsl:attribute name="content"><xsl:apply-templates select="name" /></xsl:attribute>
 </meta>
</xsl:template>

<xsl:template match="date">
 <meta name="DATE">
  <xsl:attribute name="content"><xsl:apply-templates /></xsl:attribute>
 </meta>
</xsl:template>

<xsl:template match="keywords">
 <meta name="KEYWORDS">
  <xsl:attribute name="content"><xsl:apply-templates /></xsl:attribute>
 </meta>
</xsl:template>

<xsl:template name="css" match="css">
 <style>
   <xsl:apply-templates select="document(@href)"/>
 </style>
</xsl:template>

<!-- Body ========================================================= -->

<xsl:template match="body">
 <body>
  <h1><center><xsl:value-of select="//doc/head/title/text()" /></center></h1>
  <p align="right"><i><a><xsl:attribute name="href">mailto:<xsl:value-of select="//doc/head/author/mail/text()" /></xsl:attribute><xsl:value-of select="//doc/head/author/name/text()" /></a> - <xsl:value-of select="//doc/head/date/text()" /> </i></p>
  <center>
    <xsl:call-template name="toc" />
  </center>
  <xsl:apply-templates />
 </body>
</xsl:template>

<xsl:template name="toc">
 <xsl:if test="count(intro)!=0"><a href="#intro">Introduction</a><br /></xsl:if>
 <xsl:for-each select="section">
  <a>
   <xsl:attribute name="href">#<xsl:value-of select="@nick" /></xsl:attribute>
   <xsl:value-of select="@title" />
  </a><br />
 </xsl:for-each>
 <xsl:if test="count(conc)!=0"><a href="#conc">Conclusion</a><br /></xsl:if>
 <xsl:if test="count(refs)!=0"><a href="#refs">Rfrences</a><br /></xsl:if>
</xsl:template>

<xsl:template match="section">
 <!-- Title -->
 <p>
 <a>
  <!-- Headering -->
  <xsl:attribute name="name"><xsl:value-of select="@nick" /></xsl:attribute>
  <xsl:variable name="level"><xsl:value-of select="count(ancestor::node())-2" /></xsl:variable>
  <xsl:element name="h{$level}"> 
    <!-- Numbering -->
    <xsl:number count="section" level="multiple" format="I. 1. a. " />
    <xsl:value-of select="@title" />
  </xsl:element>
 </a>
 <!-- TOC -->
 <p>
 <xsl:if test="not(@toc='no')">
   <xsl:call-template name="toc" />
 </xsl:if>
 </p>
 <xsl:apply-templates />
 </p>
</xsl:template>

<!-- Intro, Conc, Refs -->

<xsl:template match="intro">
 <a name="intro"><h1>Introduction</h1></a>
 <p><xsl:apply-templates /></p>
</xsl:template>

<xsl:template match="conc">
 <a name="conc"><h1>Conclusion</h1></a>
 <p><xsl:apply-templates /></p>
</xsl:template>

<xsl:template match="refs">
 <a name="refs"><h1>Rfrences</h1></a>
 <ul>
   <xsl:apply-templates />
 </ul>
</xsl:template>

<xsl:template match="ref">
 <li>
   <b><xsl:apply-templates select="url" /></b> : <xsl:apply-templates select="desc" />
 </li>
</xsl:template>

<xsl:template match="source">
 <xsl:copy-of select="document(@href)/html/body/*" />
</xsl:template>

<!-- HTML ========================================================= -->

<xsl:template match="b"> <b><xsl:apply-templates /></b> </xsl:template>
<xsl:template match="i"> <i><xsl:apply-templates /></i> </xsl:template>
<xsl:template match="ul"> <ul><xsl:apply-templates /></ul> </xsl:template>
<xsl:template match="ol"> <ol><xsl:apply-templates /></ol> </xsl:template>
<xsl:template match="li"> <li><xsl:apply-templates /></li> </xsl:template>
<xsl:template match="p"> <p><xsl:apply-templates /></p> </xsl:template>
<xsl:template match="code"> <code><xsl:apply-templates /></code> </xsl:template>
<xsl:template match="center"> <center><xsl:apply-templates /></center> </xsl:template>

<xsl:template match="url"> 
 <a>
   <xsl:attribute name="href">
     <xsl:choose>
       <xsl:when test="@href!=''">
         <xsl:value-of select="@href" />
       </xsl:when>
       <xsl:otherwise>
         <xsl:value-of select="." />
       </xsl:otherwise>
      </xsl:choose>
   </xsl:attribute>
  <xsl:choose>
   <xsl:when test="@desc!=''">
    <xsl:value-of select="@desc" />
   </xsl:when>
   <xsl:otherwise>
    <xsl:value-of select="." />
   </xsl:otherwise>
  </xsl:choose>
 </a> 
</xsl:template>

<xsl:template match="img">
<xsl:choose>
<xsl:when test="@center='no'">
 <img>
  <xsl:attribute name="src"><xsl:value-of select="@src" /></xsl:attribute>
  <xsl:attribute name="alt"><xsl:value-of select="@alt" /></xsl:attribute>
 </img>
</xsl:when>
<xsl:otherwise>
 <center>
 <img>
  <xsl:attribute name="src"><xsl:value-of select="@src" /></xsl:attribute>
  <xsl:attribute name="alt"><xsl:value-of select="@alt" /></xsl:attribute>
 </img>
 </center>
</xsl:otherwise> 
</xsl:choose>
</xsl:template>

<xsl:template match="CODE">
<div class="code">
<pre>
 <xsl:apply-templates />
</pre>
</div>
</xsl:template>

<!-- Code generated with VIM _6.0_ in CDATA -->

<xsl:template match="font">
 <font>
  <xsl:attribute name="color"><xsl:value-of select="@color" /></xsl:attribute>
  <xsl:apply-templates />
 </font>
</xsl:template>



</xsl:stylesheet>

